class Publication < ActiveRecord::Base
  include Rails.application.routes.url_helpers
  include ActionView::Helpers::NumberHelper

  # Taggings
  acts_as_taggable

  # Contador de visitas
  is_visitable accept_ip: true

  # URL amigáveis através do :name
  extend FriendlyId
  friendly_id :title, use: [:slugged, :history]

  # Escopos
  scope :with_locale, lambda{ |locale = I18n.locale| where("language = ?", locale) }
  scope :active     , lambda{ where(:published => true)     }
  scope :inactive   , lambda{ where(:published => false)    }
  scope :ordained   , lambda{ order("position asc").order("updated_at desc") }

  scope :list_all   , lambda{ with_locale.active.ordained   }
  scope :recents    , lambda{ |limit = 10| active.ordained.limit(limit) }

  # Callbacks
  before_save :issuu_upload
  before_destroy :issuu_delete

  attr_accessor :file

  def thumb_path
    if self.document?
      "http://image.issuu.com/#{self.document}/jpg/page_1_thumb_medium.jpg"
    else
      "/assets/missing/thumb.png"
    end
  end

  #Jquery File Uploads
  def to_jq_upload
    {
      "id"            => read_attribute(:id),
      "name"          => read_attribute(:title),
      "description"   => read_attribute(:description),
      "language"      => read_attribute(:language),
      "document"      => read_attribute(:document),
      "content_type"  => read_attribute(:content_type),
      "size"          => read_attribute(:file_size),
      "published"     => read_attribute(:published),
      "tag_list"      => read_attribute(:tag_list),
      "updated_at"    => read_attribute(:updated_at),
      "created_at"    => read_attribute(:created_at),
      "thumb"         => self.thumb_path,
      "view_url"      => publication_path(self),
      "edit_url"      => admin_publication_path(self),
      "delete_url"    => admin_publication_path(self),
      "delete_type"   => "DELETE",
      "i18n" => {
        "updated_at"  => I18n.l(read_attribute(:updated_at), format: :long),
        "created_at"  => I18n.l(read_attribute(:created_at), format: :long),
      }
    }
  end

  private

  # Refaz a ordenação dos uploads
  def self.reorder_positions(ids = nil)
    [] if ids.blank?
    counter = 1
    ActiveRecord::Base.establish_connection
    ids.each do |id|
      ActiveRecord::Base.connection.execute("UPDATE publications SET position=#{counter} WHERE id='#{id}';\n")
      counter += 1
    end
    return true
  end

  def issuu_upload
    if defined?(self.file) and self.document_name.nil?
      Rails.logger.info "\n==== Issuu Upload:"
      self.language = I18n.locale.to_s if self.language.nil?
      self.document_name     = self.slug[/^(.{0,50})/]

      language = case self.language
        when "br" then "pt"
        else self.language
      end

      params = {
        name: self.document_name,
        title: self.title,
        tags: self.tag_list,
        description: self.description,
        language: language,
        downloadable: true
      }
      begin
        document = Issuu::Document.upload self.file.tempfile, self.file.content_type, params

        self.attributes= {
          document: document.documentId,
          username: document.username
          }
      rescue Issuu::IssuuException, Timeout::Error => msg
        errors.add :document, msg.to_s
        document = { error: true, msg: msg }
      end
      Rails.logger.info "\n  #{document.inspect}\n"
    end
  end

  def issuu_delete
    begin
      Issuu::Document.delete [self.document_name]
    rescue Issuu::IssuuException, Timeout::Error => msg
      { error: true, msg: msg }
    end
  end
end
